{% extends "layout.html" %}
{% set title = 'Overview' %}
{% set css_files = css_files + ["_static/force/force.css"] %}
{% block body %}


<h2>High productivity software for complex networks</a></h2>

<blockquote>
<p>NetworkX is a Python language software package for the creation, manipulation, and
study of the structure, dynamics, and functions of complex networks.</p>
</blockquote>

<h2>Quick Example</h2>



<table align="center" style="margin-left: 20px"><tr>
    <td width="50%">
<div class="highlight-python"><div class="highlight"><pre><span class="gp">&gt;&gt;&gt; </span><span class="k">import</span> <span class="nn">networkx</span> <span class="k">as</span> <span class="nn">nx</span>

<span class="gp">&gt;&gt;&gt; </span><span class="n">G</span><span class="o">=</span><span class="n">nx</span><span class="o">.</span><span class="n">Graph</span><span class="p">()</span>
<span class="gp">&gt;&gt;&gt; </span><span class="n">G</span><span class="o">.</span><span class="n">add_node</span><span class="p">(</span><span class="s">&quot;spam&quot;</span><span class="p">)</span>
<span class="gp">&gt;&gt;&gt; </span><span class="n">G</span><span class="o">.</span><span class="n">add_edge</span><span class="p">(</span><span class="mf">1</span><span class="p">,</span><span class="mf">2</span><span class="p">)</span>
<span class="gp">&gt;&gt;&gt; </span><span class="k">print</span><span class="p">(</span class="p"><span class="n">G</span><span class="o">.</span><span class="n">nodes</span><span class="p">())</span>
<span class="go">[1, 2, &#39;spam&#39;]</span>
<span class="gp">&gt;&gt;&gt; </span><span class="k">print</span><span class="p">(</span class="p"><span class="n">G</span><span class="o">.</span><span class="n">edges</span><span class="p">())</span>
<span class="go">[(1, 2)]</span>
</pre></div>
    </td>
    <td width="50%">
    <div id="chart"></div>
    <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/d3/3.4.11/d3.min.js"></script>
    <script type="text/javascript" src="_static/force/force.js"></script>
</td></tr>
</blockquote>
</table>


<h2>Documentation</h2>

<table class="contentstable" align="center" style="margin-left: 30px"><tr>
  <td width="50%">

    <p class="biglink"><a class="biglink" href="{{ pathto("overview") }}">Overview</a><br/>
       <span class="linkdescr">the big picture</span></p>

    <p class="biglink"><a class="biglink" href="{{ pathto("tutorial/index") }}">Tutorial</a><br/>
       <span class="linkdescr">get started here</span></p>
    <p class="biglink"><a class="biglink" href="{{ pathto("reference/index") }}">Reference</a><br/>
       <span class="linkdescr">guide to all functions and classes</span></p>
  </td>
  <td width="50%">
    <p class="biglink"><a class="biglink" href="{{ pathto("contents") }}">Contents</a><br/>
       <span class="linkdescr">all documentation</span></p>
    <p class="biglink"><a class="biglink" href="{{ pathto("examples/index") }}">Examples</a><br/>
       <span class="linkdescr">using the library</span></p>
    <p class="biglink"><a class="biglink" href="{{ pathto("gallery") }}">Gallery</a><br/>
       <span class="linkdescr">network drawings</span></p>
  </td></tr>
</table>

<h2>Features</h2>
<ul class="simple">
<li> Python language data structures for graphs, digraphs, and multigraphs.
<li> Nodes can be &quot;anything&quot; (e.g. text, images, XML records)</li>
<li> Edges can hold arbitrary data (e.g. weights, time-series) 
<li> Generators for classic graphs, random graphs, and synthetic networks</li>
<li> Standard graph algorithms</li>
<li> Network structure and analysis measures</li>
<li> Basic graph drawing</li>
<li> Open source <a href="reference/legal.html">BSD license</a> </li>
<li> Well tested: more than 1500 unit tests</li>
<li> Additional benefits from Python: fast prototyping, easy to teach, multi-platform</li>
</ul>

<p><em></em></p>


{% endblock %}
