{% extends "!layout.html" %}

{% block rootrellink %}
        <li><a href="http://networkx.github.com/">NetworkX Home </a> |&nbsp;</li>
        <li><a href="http://networkx.github.com/documentation.html">Documentation </a>|&nbsp;</li>
        <li><a href="http://networkx.github.com/download.html">Download </a> |&nbsp;</li>
        <li><a href="http://github.com/networkx">Developer (Github)</a></li>


{% endblock %}

{% block relbar1 %}
<div style="color: black;background-color: white; font-size: 3.2em; text-align: left; padding: 15px 10px 10px 15px">
NetworkX
</div>
{{ super() }}
{% if version.split('.')[-1].startswith('dev') %}
<p>This documentation is for the development version <b>{{ version }}</b></p>
{% endif %}
{% endblock %}

{# put the sidebar before the body #}
{% block sidebar1 %}
{{ sidebar() }}{% endblock %}
{% block sidebar2 %}{% endblock %}
