@import url('css/theme.css');

.wy-table-responsive table td, .wy-table-responsive table th {
    white-space: normal;
}

.wy-table-responsive table td:first-child {
    white-space: nowrap;
}

code, .rst-content tt, .linenodiv pre, div[class^="highlight"] pre {
    font-family: 'Inconsolata', 'Monaco', 'Consolas', monospace;
    font-size: 85%;
}
