"""
Version information for NetworkX, created during installation.

Do not add this file to the repository.

"""

import datetime

version = '1.9.1'
date = 'Sat Sep 20 13:09:35 2014'

# Was NetworkX built from a development version? If so, remember that the major
# and minor versions reference the "target" (rather than "current") release.
dev = False

# Format: (name, major, min, revision)
version_info = ('networkx', '1', '9.1', None)

# Format: a 'datetime.datetime' instance
date_info = datetime.datetime(2014, 9, 20, 13, 9, 35, 734061)

# Format: (vcs, vcs_tuple)
vcs_info = (None, (None, None))

